/**
 * EasyBeans
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CommonMethodMetadata.java 4389 2008-12-15 13:48:57Z alitokmen $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.common.impl;

import org.ow2.util.ee.metadata.common.api.ICommonClassMetadata;
import org.ow2.util.ee.metadata.common.api.ICommonFieldMetadata;
import org.ow2.util.ee.metadata.common.api.ICommonMethodMetadata;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationResource;
import org.ow2.util.ee.metadata.common.api.struct.IJEjbEJB;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceContext;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceUnit;
import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;
import org.ow2.util.ee.metadata.common.api.struct.IJwsHandlerChain;
import org.ow2.util.scan.api.metadata.MethodMetadata;
import org.ow2.util.scan.api.metadata.structures.JMethod;

/**
 * EZB common method metadata.
 * @author Gael Lalire
 * @param <C> class type
 * @param <M> method type
 * @param <F> field type
 */
public class CommonMethodMetadata<
    C extends ICommonClassMetadata<C, M, F>,
    M extends ICommonMethodMetadata<C, M, F>,
    F extends ICommonFieldMetadata<C, M, F>>
        extends MethodMetadata<C, M, F> implements ICommonMethodMetadata<C, M, F> {

    /**
     * metadata common with class, field and method.
     */
    private AbsSharedMetadata commonMetadata;

    /**
     * PostConstruct method ?
     */
    private boolean postConstruct = false;

    /**
     * PreDestroy method ?
     */
    private boolean preDestroy = false;

    /**
     * Constructor.
     * @param jMethod signature
     * @param classMetadata parent
     */
    protected CommonMethodMetadata(final JMethod jMethod, final C classMetadata) {
        super(jMethod, classMetadata);
        this.commonMetadata = new AbsSharedMetadata();
    }

    /**
     * @return JEjbEJB object representing javax.ejb.EJB annotation.
     */
    public final IJEjbEJB getJEjbEJB() {
        return this.commonMetadata.getJEjbEJB();
    }

    /**
     * Set JAnnotationEJB object.
     * @param jEjbEJB object representing javax.annotation.EJB annotation.
     */
    public final void setJEjbEJB(final IJEjbEJB jEjbEJB) {
        this.commonMetadata.setJEjbEJB(jEjbEJB);
    }

    /**
     * @return JAnnotationResource object representing javax.annotation.Resource
     *         annotation.
     */
    public final IJAnnotationResource getJAnnotationResource() {
        return this.commonMetadata.getJAnnotationResource();
    }

    /**
     * Set JAnnotationResource object.
     * @param jAnnotationResource object representing javax.annotation.Resource
     *        annotation.
     */
    public final void setJAnnotationResource(final IJAnnotationResource jAnnotationResource) {
        this.commonMetadata.setJAnnotationResource(jAnnotationResource);
    }

    /**
     * @return true if this field is used as a persistence context.
     */
    public final boolean isPersistenceContext() {
        return this.commonMetadata.isPersistenceContext();
    }

    /**
     * @return the persistence context infos.
     */
    public final IJavaxPersistenceContext getJavaxPersistenceContext() {
        return this.commonMetadata.getJavaxPersistenceContext();
    }

    /**
     * Sets the persistence context info on this field.
     * @param javaxPersistenceContext information on persistence context.
     */
    public final void setJavaxPersistenceContext(final IJavaxPersistenceContext javaxPersistenceContext) {
        this.commonMetadata.setJavaxPersistenceContext(javaxPersistenceContext);
    }

    /**
     * @return true if this field is used as a persistence unit.
     */
    public final boolean isPersistenceUnit() {
        return this.commonMetadata.isPersistenceUnit();
    }

    /**
     * @return the persistence unit infos.
     */
    public final IJavaxPersistenceUnit getJavaxPersistenceUnit() {
        return this.commonMetadata.getJavaxPersistenceUnit();
    }

    /**
     * Sets the persistence unit info on this field.
     * @param javaxPersistenceUnit information on persistence unit.
     */
    public final void setJavaxPersistenceUnit(final IJavaxPersistenceUnit javaxPersistenceUnit) {
        this.commonMetadata.setJavaxPersistenceUnit(javaxPersistenceUnit);
    }

    /**
     * @return true if method has &#64;{@link javax.annotation.PostConstruct}.
     */
    public boolean isPostConstruct() {
        return postConstruct;
    }

    /**
     * Sets true if method has &#64;{@link javax.annotation.PostConstruct}.
     * @param postConstruct true/false.
     */
    public void setPostConstruct(final boolean postConstruct) {
        this.postConstruct = postConstruct;
    }

    /**
     * @return true if method has &#64;{@link javax.annotation.PreDestroy}.
     */

    public boolean isPreDestroy() {
        return preDestroy;
    }

    /**
     * Sets true if method has &#64;{@link javax.annotation.PreDestroy}.
     * @param preDestroy true/false.
     */
    public void setPreDestroy(final boolean preDestroy) {
        this.preDestroy = preDestroy;
    }

    /**
     * @return IJWebServiceRef object representing javax.xml.ws.WebServiceRef annotation.
     */
    public IJaxwsWebServiceRef getJaxwsWebServiceRef() {
        return commonMetadata.getJaxwsWebServiceRef();
    }

    /**
     * Set JWebServiceRef object.
     *
     * @param wsr object representing javax.xml.ws.WebServiceRef annotation.
     */
    public void setJaxwsWebServiceRef(final IJaxwsWebServiceRef wsr) {
        commonMetadata.setJaxwsWebServiceRef(wsr);
    }

    /**
     * @return IJwsHandlerChain object representing &#64;{@link javax.jws.HandlerChain} annotation.
     */
    public IJwsHandlerChain getJwsHandlerChain() {
        return commonMetadata.getJwsHandlerChain();
    }

    /**
     * Set JwsHandlerChain object.
     * @param jwshandlerChain object representing &#64;{@link javax.jws.HandlerChain} annotation.
     */
    public void setJwsHandlerChain(final IJwsHandlerChain jwshandlerChain) {
        commonMetadata.setJwsHandlerChain(jwshandlerChain);
    }
}
